import logging
import unittest
import shutil
from pathlib import Path

from chunk_files import chunk_files


BASE_PATH = Path(__file__).resolve().parent


raw_dir = BASE_PATH / 'raw'
output_dir = BASE_PATH / 'output'


class MountMock:
    def __init__(self,):
        self.mount_point = str(raw_dir)


class DatasetMountMock:
    def __enter__(self):
        return MountMock()

    def __exit__(__exit__, arg1, arg2, arg3, arg4=None):
        print("__exit__")


class DatasetMock:

    def mount(self):
        mnt = DatasetMountMock()
        return mnt

class TestFiltserData(unittest.TestCase):
    # Set log level to info
    logging.getLogger().setLevel(logging.INFO)

    def tearDown(self):
       """clean output dir"""
       if output_dir.is_dir():
           shutil.rmtree(str(output_dir))

    def test_chunk_files(self):

        chunk_files(DatasetMock(), str(output_dir), 4, False)

        dir_list = [p for p in Path(str(output_dir)).iterdir() if p.is_dir()]
        self.assertEqual(
            4,
            len(dir_list)
        )



import argparse
import json
from pathlib import Path

from azureml.core import Dataset, Run
from chunk_files import chunk_files
import shutil

# Get parameters
parser = argparse.ArgumentParser()
parser.add_argument("--datasets_json", type=str, help="Dataset json")
parser.add_argument(
    "--raw_data_dir", type=str, help="Directory to store raw pdf,png,jpeg,tiff"
)
parser.add_argument("--number_chunk", type=int, help="Number of chunk")
parser.add_argument("--chunk_index", type=int, help="Chunk index")

# Get arguments from parser
args = parser.parse_args()
datasets = json.loads(args.datasets_json)
raw_data_dir = args.raw_data_dir
number_chunk = args.number_chunk
chunk_index = args.chunk_index

# Get the experiment run context
run = Run.get_context()
workspace = run.experiment.workspace

# Get raw data
for dataset in datasets:
    dataset_name = dataset["name"]
    dataset_version = dataset["version"]
    dataset_directory = dataset["directory"]
    raw_dataset = Dataset.get_by_name(workspace, dataset_name, version=dataset_version)
    print(
        f"Retrieve raw data in dataset: {raw_dataset.name} - version: {raw_dataset.version}"
    )
    if number_chunk == 1:
        chunk_name = "chunk0"
        if dataset_directory is None or dataset_directory == "":
            dir_path = Path(str(Path(raw_data_dir) / chunk_name))
        else:
            dir_path = Path(str(Path(raw_data_dir) / chunk_name / dataset_directory))
        dataset.download(target_path=dir_path, overwrite=False)
    else:
        raw_dataset.as_mount("/mnt/input_data")
        chunk_files(raw_dataset, raw_data_dir, number_chunk, shutil.copy, [chunk_index], dataset_directory)





import sys
import time
from pathlib import Path
import shutil
import numpy
import multiprocessing as mp
import gc


def move_files(source_dir, chunk, dest_dir, chunk_name, sub_directory: str = None):
    for file_path in chunk:
        try:
            file_pathlib = Path(file_path)
            filename = file_pathlib.stem + file_pathlib.suffix
            if sub_directory is None or sub_directory == "":
                dest_path = Path(dest_dir) / chunk_name / filename
            else:
                dest_path = Path(dest_dir) / chunk_name / sub_directory / filename
            scr_dir = str(Path(source_dir) / filename)
            if Path(scr_dir).is_file():
                shutil.copy(scr_dir, str(dest_path))
            else:
                print("a new file appear, won't be copied")
        except Exception as ex:
            print("Exception thrown. x does not exist." + str(file_path) + " " + str(ex))


def chunk_files(raw_dataset, dest_dir: str,
                number_chunk: int, multithread=True, chunk_index_to_copy=None, sub_directory: str = None):
    with raw_dataset.mount() as mount_context:
        all_documents_flatten_dir = mount_context.mount_point
        files_list = [str(p) for p in Path(all_documents_flatten_dir).iterdir() if p.is_file()]
    files_list.sort()
    number_files = len(files_list)
    print("number file is : " + str(number_files))
    chunks = numpy.array_split(numpy.array(files_list), number_chunk)
    for index, chunk in enumerate(chunks):
        if chunk_index_to_copy is not None and index not in chunk_index_to_copy:
            continue
        chunk_name = "chunk" + str(index)
        if sub_directory is None or sub_directory == "":
            dir_path = Path(str(Path(dest_dir) / chunk_name))
        else:
            dir_path = Path(str(Path(dest_dir) / chunk_name / sub_directory))
        dir_path.mkdir(parents=True, exist_ok=True)
        start = time.time()

        if multithread:
            number_sub_chunk = int(len(chunk) / 5000)
            if number_sub_chunk == 0:
                number_sub_chunk = 1
            sub_chunks = numpy.array_split(numpy.array(chunk), number_sub_chunk)
            for sub_chunk in sub_chunks:
                with raw_dataset.mount() as mount_context:
                    all_documents_flatten_dir = mount_context.mount_point
                    number_cpus = mp.cpu_count()
                    print('Number cores found: ' + str(number_cpus))
                    pool = mp.Pool(number_cpus)
                    csv_files_chuncks = numpy.array_split(sub_chunk, number_cpus)
                    try:
                        pool.starmap(move_files, [(all_documents_flatten_dir, row, dest_dir, chunk_name, sub_directory) for row in
                                                  csv_files_chuncks])
                    except Exception:
                        print("Unexpected error:", sys.exc_info()[0])
                        raise
                    finally:
                        pool.close()
                        gc.collect()
        else:
            move_files(all_documents_flatten_dir, chunk, dest_dir, chunk_name)
            gc.collect()

        end = time.time() - start
        print("Time move chunk " + str(index) + " " + str(end))



import argparse
import json
from pathlib import Path
from azureml.core import Run, Dataset
from azureml.core.datastore import Datastore
from azureml.data.datapath import DataPath

# Get parameters
parser = argparse.ArgumentParser()
parser.add_argument(
    '--img_dir', type=str, help='Directory where images to save are stored'
)
parser.add_argument('--save_datastore_name', type=str, help='save datastore name')
parser.add_argument('--save_datastore_path', type=str, help='save datastore path')
parser.add_argument("--save_datasets_json", type=str, help="Dataset json")

parser.add_argument('--chunk_index', type=str, help='index of the current chunk')

# Get arguments from parser
args = parser.parse_args()
img_dir = args.img_dir
# datastore_name = args.datastore_name
chunk_index = args.chunk_index
save_datastore_name = args.save_datastore_name
save_datastore_path = args.save_datastore_path
save_datasets = json.loads(args.save_datasets_json)

# Get the experiment run context
run = Run.get_context()
workspace = run.experiment.workspace

# Load data
print("Loading Data...")
data_output_dir_path = Path(img_dir)

# summary_filename = "summary_chunk" + str(chunk_index) + ".json"
chunk_name = "chunk" + str(chunk_index)
json_chunk_name = "json" + chunk_name
output_jsons = str(data_output_dir_path / json_chunk_name)
# create_summary(output_jsons, str(data_output_dir_path / summary_filename))

datastore = Datastore.get(workspace, save_datastore_name)
dataset = Dataset.File.upload_directory(
    src_dir=str(data_output_dir_path), target=DataPath(datastore, save_datastore_path), pattern="*", show_progress=True,
    overwrite=True
)

output_jsons_path = Path(output_jsons)
if output_jsons_path.is_dir():
    dir_list = [p for p in Path(output_jsons).iterdir() if p.is_dir()]

    for dir_path in dir_list:
        dir_name = dir_path.name
        source = str(output_jsons_path / dir_name)
        dummy_path = str(output_jsons_path / dir_name / 'dummy.json')

        subdir_list = [p for p in Path(source).iterdir() if p.is_dir()]
        subfiles_list = [p for p in Path(source).iterdir() if p.is_file()]
        if len(subfiles_list) <= 0 < len(subdir_list):
            Dataset.File.upload_directory(
                src_dir=source, target=DataPath(datastore, str(Path(save_datastore_path) / dir_name)), pattern="*",
                show_progress=True,
                overwrite=True
            )

        for subdir_path in subdir_list:
            subdirname = subdir_path.name
            destination = str(Path(save_datastore_path) / dir_name / subdirname)
            sub_source = str(Path(subdir_path) / dir_name / subdirname)
            Dataset.File.upload_directory(
                src_dir=sub_source, target=DataPath(datastore, destination), pattern="*",
                show_progress=True,
                overwrite=True
            )

for dataset in save_datasets:
    file_data_set = Dataset.File.from_files(path=(datastore, save_datastore_path + '/' + dataset["pattern"]))
    registered_dataset = file_data_set.register(
        workspace=workspace,
        name=dataset["name"],
        description=dataset["description"],
        tags={},
        create_new_version=True,
    )
    print("Registered dataset: " + registered_dataset.name + ' version: ' + str(registered_dataset.version))

# End the run
run.complete()
